#include "refobj.h"
#include "stmt.h"
#include "varstore.h"
#include "parser.h"

void header() {
    printf("    .global main\n");
    printf("    .text\n");
    printf("main:\n");
    printf("    pushq    %%rbp\n");
}

void footer() {
    printf("    movq    $0, %%rax\n");
    printf("    popq    %%rbp\n");
    printf("    ret\n");

    printf("    .section .rodata\n");
    printf("ANSW:       .string \"answer: %%ld\\n\"\n");
    printf("ENTR:       .string \"enter: \"\n");
    printf("ENTR_FMT:   .string \"%%ld\"\n");
    
    printf("    .data\n");
    printf("VAR:        .zero %d\n", var_store_size()*8);
}

int main(int argc, char **argv) {
    char *fname = NULL;
    if(argc == 1)
        fname = NULL;
    else if(argc == 2)
        fname = argv[1];
    else
        ON_FALSE_EXIT(0, strmsg("Usage: %s [program_file]", argv[0]));

    //parse program
    stmt_t *stmt = parse(fname);
    //stmt->print(stmt);

    //init variable store
    var_store_init();

    //compile
    header();
    stmt->exec(stmt);
    footer();

    //destroy variable store
    var_store_destroy();

    //release statement
    stmt->ref.release(&stmt->ref);
    refobj_check_dealloc();
    
    return 0;
}
